#ident	"$Header: /var/cvsroot/htdb/src/dso/io.c,v 1.4 2006/01/08 00:20:15 del Exp $"

/*
	Copyright  1994-2006 David Whittemore
	<del@adjective.com>, all rights reserved.

	Permission to use, copy, modify, and non-commercially distribute
	this software and its documentation for any purpose is hereby granted
	without fee, provided that the above copyright notice appear in all
	copies and that both that copyright notice and this permission notice
	appear in supporting documentation.  No representations are made about
	the suitability of this software for any purpose.  It is provided
	"as is" without express or implied warranty.
 */
#include	"htdb.h"

#define	SYNTAX_fopen	"usage: \\$\\{dso_fopen(handle, path, mode)\\}"
char    *dso_fopen(char *in)
{
	char	**args;
	int		num_args = htdb_splitArgs(in, ",", &args, 3);

	if (num_args == 3) {
		char	*prefix = args[0],
				*path = args[1],
				*mode = args[2];
		spaceData	*b = file_lookup(prefix);

		if (b == (spaceData *)NULL) {
			filep_t	*fpt = (filep_t *)malloc(sizeof(filep_t));
			/*
				new file request
			 */
			fpt->path = strdup(path);
			fpt->mode = strdup(mode);
			(FILE *)fpt->fp = (FILE *)fopen(fpt->path, fpt->mode);
			file_store(prefix, (void *)fpt);
		} else {
			/*
				file's already open
			 */
		}

		htdb_deleteArgs(args, num_args);
		return DSO_RETURNS_SUCCESS("");
	} else {
		htdb_deleteArgs(args, num_args);
		return DSO_RETURNS_SUCCESS_CHALLENGED(SYNTAX_fopen);
	}
}

#define	SYNTAX_fclose	"usage: \\$\\{dso_fclose(handle)\\}"
char    *dso_fclose(char *in)
{
	char	*prefix = in;

	if (prefix && *prefix) {
		spaceData	*b = file_lookup(prefix);

		if (b == (spaceData *)NULL) {
			return DSO_RETURNS_SUCCESS_CHALLENGED(SYNTAX_fclose);
		} else {
			/*
				close the file, clean up.
			 */
			htdb_destroyFilePointer(b->value);
			b->value = NULL;
			free(b->key);
			b->key = strdup("");
			return DSO_RETURNS_SUCCESS("");
		}

	} else {
		return DSO_RETURNS_SUCCESS_CHALLENGED(SYNTAX_fclose);
	}
}

#define	SYNTAX_fwrite	"usage: \\$\\{dso_fwrite(handle, string)\\}"
char    *dso_fwrite(char *in)
{
	char	**args;
	int		num_args = htdb_splitArgs(in, ",", &args, 2);

	if (num_args == 2) {
		char	*prefix = args[0],
				*string = args[1];
		spaceData	*b = file_lookup(prefix);

		if (b == (spaceData *)NULL) {
			htdb_deleteArgs(args, num_args);
			return DSO_RETURNS_SUCCESS_CHALLENGED(SYNTAX_fwrite);
		} else {
			filep_t	*fpt = (filep_t *)b->value;
			fprintf((FILE *)fpt->fp, string);
		}

		htdb_deleteArgs(args, num_args);
		return DSO_RETURNS_SUCCESS("");
	} else {
		htdb_deleteArgs(args, num_args);
		return DSO_RETURNS_SUCCESS_CHALLENGED(SYNTAX_fwrite);
	}
}

#define	SYNTAX_wget	"usage: wget(prefix, URL)"

char	*dso_wget(char *in)
{
	char 	**args;
	int		num_args = htdb_splitArgs(in, ",", &args, 2);

	if (num_args == 2) {
		char	*prefix = args[0],
				*url = args[1];

		if (BOOL_TRUE == htdb_wget(prefix, url, WGET_DEFAULT_TIMEOUT)) {
			htdb_deleteArgs(args, num_args);
			return DSO_RETURNS_SUCCESS("");
		} else {
			htdb_deleteArgs(args, num_args);
			return DSO_RETURNS_SUCCESS_CHALLENGED("");
		}
	} else {
		htdb_deleteArgs(args, num_args);
		return DSO_RETURNS_SUCCESS_CHALLENGED(SYNTAX_wget);
	}
}

char	*dso_rss(char *in)
{
	char 	**args;
	int		num_args = htdb_splitArgs(in, ",", &args, 0);
	int_t	timeout = WGET_DEFAULT_TIMEOUT;
	bool_t	status = BOOL_FALSE;

	if (num_args > 0) {
		if (num_args == 2) {
			timeout = atoi(args[1]);
		}
		if (timeout <= 0)
			timeout = WGET_DEFAULT_TIMEOUT;
		{
			if (BOOL_TRUE == htdb_wget(args[0], args[0], timeout)) {
				status = htdb_parseXML(args[0], htdb_getptrval(args[0], "body"));
			}
		}
	}
	htdb_deleteArgs(args, num_args);
	if (BOOL_TRUE == status)
		return DSO_RETURNS_SUCCESS("");
	else
		return DSO_RETURNS_SUCCESS_CHALLENGED("");
}

char	*dso_cleanupRSS(char *in)
{
	htdb_clearXML(in);
}

char	*dso_cleanupXML(char *in)
{
	htdb_clearXML(in);
}

#define	SYNTAX_fileExists	"fileExists: no such file"

/*
	dso_fileExists()
		make a filesystem call to see if file exists
 */
char	*dso_fileExists(char *in)
{
	struct	stat    buf;
	if (stat(in, &buf) == 0)
		return DSO_RETURNS_SUCCESS(GENERIC_SUCCESS_STRING);
	return DSO_RETURNS_SUCCESS_CHALLENGED(SYNTAX_fileExists);
}

#define SYNTAX_system	"usage: \\$\\{system(command)\\}"

/*
	dso_system()
		makes a system call as whatever user the process is running as.
		begeesus.  i can't believe i'm including this..
		we try to do *some* damage control by not allowing root processes.
 */
char	*dso_system(char *source)
{
	if (BOOL_TRUE == amRoot())
		return DSO_RETURNS_SUCCESS_CHALLENGED(ARE_YOU_INSANE);
	else {
		return DSO_RETURNS_SUCCESS_ASIS(htdb_system(source));
	}
}

#define	SENDMAIL_SYNTAX		"usage: \\$\\{sendmail(message)\\}"
#define	SENDMAIL_MISSING	"sendmail undefined in config.htdb"

/*
	dso_sendmail()
		pipelines the input string through whatever resourse
		is defined as `confSendmail'
 */
char	*dso_sendmail(char *source)
{
	if (BOOL_TRUE == amRoot())
		return DSO_RETURNS_SUCCESS_CHALLENGED(ARE_YOU_INSANE);
	else {
		return (sendmail(htdb_getval("confSendmail"), source) == 0) ?
			DSO_RETURNS_SUCCESS_CHALLENGED(SENDMAIL_MISSING) :
			DSO_RETURNS_SUCCESS(GENERIC_SUCCESS_STRING);
	}
}

